@@ -2,6 +2,11 @@ import argparse
 import sys
 import warnings
 from gettext import gettext
+from typing import Any
+from typing import Dict
+from typing import List
+from typing import Optional
+from typing import Tuple
 
 import py
 
@@ -21,12 +26,12 @@ class Parser:
 
     def __init__(self, usage=None, processopt=None):
         self._anonymous = OptionGroup("custom options", parser=self)
-        self._groups = []
+        self._groups = []  # type: List[OptionGroup]
         self._processopt = processopt
         self._usage = usage
-        self._inidict = {}
-        self._ininames = []
-        self.extra_info = {}
+        self._inidict = {}  # type: Dict[str, Tuple[str, Optional[str], Any]]
+        self._ininames = []  # type: List[str]
+        self.extra_info = {}  # type: Dict[str, Any]
 
     def processoption(self, option):
         if self._processopt:
@@ -42,7 +47,7 @@ class Parser:
 
         The returned group object has an ``addoption`` method with the same
         signature as :py:func:`parser.addoption
-        <_pytest.config.Parser.addoption>` but will be shown in the
+        <_pytest.config.argparsing.Parser.addoption>` but will be shown in the
         respective group in the output of ``pytest. --help``.
         """
         for group in self._groups:
@@ -77,10 +82,10 @@ class Parser:
 
         self.optparser = self._getparser()
         try_argcomplete(self.optparser)
-        args = [str(x) if isinstance(x, py.path.local) else x for x in args]
-        return self.optparser.parse_args(args, namespace=namespace)
+        strargs = [str(x) if isinstance(x, py.path.local) else x for x in args]
+        return self.optparser.parse_args(strargs, namespace=namespace)
 
-    def _getparser(self):
+    def _getparser(self) -> "MyOptionParser":
         from _pytest._argcomplete import filescompleter
 
         optparser = MyOptionParser(self, self.extra_info, prog=self.prog)
@@ -94,7 +99,10 @@ class Parser:
                     a = option.attrs()
                     arggroup.add_argument(*n, **a)
         # bash like autocompletion for dirs (appending '/')
-        optparser.add_argument(FILE_OR_DIR, nargs="*").completer = filescompleter
+        # Type ignored because typeshed doesn't know about argcomplete.
+        optparser.add_argument(  # type: ignore
+            FILE_OR_DIR, nargs="*"
+        ).completer = filescompleter
         return optparser
 
     def parse_setoption(self, args, option, namespace=None):
@@ -103,19 +111,21 @@ class Parser:
             setattr(option, name, value)
         return getattr(parsedoption, FILE_OR_DIR)
 
-    def parse_known_args(self, args, namespace=None):
+    def parse_known_args(self, args, namespace=None) -> argparse.Namespace:
         """parses and returns a namespace object with known arguments at this
         point.
         """
         return self.parse_known_and_unknown_args(args, namespace=namespace)[0]
 
-    def parse_known_and_unknown_args(self, args, namespace=None):
+    def parse_known_and_unknown_args(
+        self, args, namespace=None
+    ) -> Tuple[argparse.Namespace, List[str]]:
         """parses and returns a namespace object with known arguments, and
         the remaining arguments unknown at this point.
         """
         optparser = self._getparser()
-        args = [str(x) if isinstance(x, py.path.local) else x for x in args]
-        return optparser.parse_known_args(args, namespace=namespace)
+        strargs = [str(x) if isinstance(x, py.path.local) else x for x in args]
+        return optparser.parse_known_args(strargs, namespace=namespace)
 
     def addini(self, name, help, type=None, default=None):
         """ register an ini-file option.
@@ -163,8 +173,8 @@ class Argument:
     def __init__(self, *names, **attrs):
         """store parms in private vars for use in add_argument"""
         self._attrs = attrs
-        self._short_opts = []
-        self._long_opts = []
+        self._short_opts = []  # type: List[str]
+        self._long_opts = []  # type: List[str]
         self.dest = attrs.get("dest")
         if "%default" in (attrs.get("help") or ""):
             warnings.warn(
@@ -268,8 +278,8 @@ class Argument:
                     )
                 self._long_opts.append(opt)
 
-    def __repr__(self):
-        args = []
+    def __repr__(self) -> str:
+        args = []  # type: List[str]
         if self._short_opts:
             args += ["_short_opts: " + repr(self._short_opts)]
         if self._long_opts:
@@ -286,7 +296,7 @@ class OptionGroup:
     def __init__(self, name, description="", parser=None):
         self.name = name
         self.description = description
-        self.options = []
+        self.options = []  # type: List[Argument]
         self.parser = parser
 
     def addoption(self, *optnames, **attrs):
@@ -385,7 +395,7 @@ class MyOptionParser(argparse.ArgumentParser):
                     options = ", ".join(option for _, option, _ in option_tuples)
                     self.error(msg % {"option": arg_string, "matches": options})
                 elif len(option_tuples) == 1:
-                    option_tuple, = option_tuples
+                    (option_tuple,) = option_tuples
                     return option_tuple
             if self._negative_number_matcher.match(arg_string):
                 if not self._has_negative_number_optionals:
@@ -405,6 +415,12 @@ class DropShorterLongHelpFormatter(argparse.HelpFormatter):
     - cache result on action object as this is called at least 2 times
     """
 
+    def __init__(self, *args, **kwargs):
+        """Use more accurate terminal width via pylib."""
+        if "width" not in kwargs:
+            kwargs["width"] = py.io.get_terminal_width()
+        super().__init__(*args, **kwargs)
+
     def _format_action_invocation(self, action):
         orgstr = argparse.HelpFormatter._format_action_invocation(self, action)
         if orgstr and orgstr[0] != "-":  # only optional arguments
@@ -421,7 +437,7 @@ class DropShorterLongHelpFormatter(argparse.HelpFormatter):
         option_map = getattr(action, "map_long_option", {})
         if option_map is None:
             option_map = {}
-        short_long = {}
+        short_long = {}  # type: Dict[str, str]
         for option in options:
             if len(option) == 2 or option[2] == " ":
                 continue
